package cs108;

import java.util.*;

public final class BurrowsWheelerTransform {
    private BurrowsWheelerTransform() {}

    // Variante 1 du calcul des rotations : au moyen d'une file
    private static Queue<Character> stringToQueue(String s) {
        Queue<Character> q = new ArrayDeque<>(s.length());
        for (int i = 0; i < s.length(); i += 1) q.add(s.charAt(i));
        return q;
    }

    private static String queueToString(Queue<Character> q) {
        StringBuilder b = new StringBuilder(q.size());
        for (char c: q) b.append(c);
        return b.toString();
    }

    private static List<String> allRotations(String s) {
        List<String> rs = new ArrayList<>(s.length());
        Queue<Character> q = stringToQueue(s);
        while (rs.size() < s.length()) {
            rs.add(queueToString(q));
            q.add(q.remove());
        }
        return rs;
    }

    // Variante 2 du calcul des rotations : au moyen de `substring` et `charAt`
    private static List<String> allRotations2(String s) {
        List<String> rs = new ArrayList<>(s.length());
        while (rs.size() < s.length()) {
            rs.add(s);
            s = s.substring(1) + s.charAt(0);
        }
        return rs;
    }

    public static Pair<Integer, String> forward(String s) {
        if (s.isEmpty())
            throw new IllegalArgumentException("empty input string");

        List<String> rs = allRotations2(s);
        Collections.sort(rs);
        StringBuilder b = new StringBuilder(s.length());
        for (String r: rs) b.append(r.charAt(r.length() - 1));
        return new Pair<>(rs.indexOf(s), b.toString());
    }

    public static String backward(Pair<Integer, String> p) {
        int index = p.first();
        String s = p.second();

        if (! (0 <= index && index < s.length()))
            throw new IndexOutOfBoundsException("invalid index " + index
            + " (should be between 0 and " + (s.length() - 1) + ")");

        List<String> rs = new ArrayList<>(Collections.nCopies(s.length(), ""));
        for (int i = 0; i < s.length(); ++i) {
            for (int j = 0; j < s.length(); ++j)
                rs.set(j, s.charAt(j) + rs.get(j));
            Collections.sort(rs);
        }
        return rs.get(index);
    }
}
